from django import forms

from alert.models import AlertPreference, Alert
from alert.utils import ALERT_TYPES, ALERT_BACKENDS, BaseAlert



class AlertPreferenceForm(forms.Form):
    """
    Shows a form with a checkbox for each alert/backend combination for
    the user to choose whether or not to receive the alert through that
    backend.
    """
    
    def __init__(self, user, alerts=None, backends=None, *args, **kwargs):
        self.user = user
        
        if alerts is None: 
            alerts = ALERT_TYPES.values()
        if backends is None: 
            backends = ALERT_BACKENDS.values()
            
        self.alerts = alerts
        self.backends = backends
        
        self.prefs = AlertPreference.objects.get_user_prefs(user).items()
        
        super(AlertPreferenceForm, self).__init__(*args, **kwargs)
        
        for ((alert_type, backend), pref) in self.prefs:
            if alert_type not in self.alerts or backend not in self.backends: 
                continue
            
            attr = self._field_id(alert_type, backend)
            alert = ALERT_TYPES[alert_type]
            self.fields[attr] = forms.BooleanField(
                                                   label=alert.title, 
                                                   help_text=alert.description,
                                                   initial=pref,
                                                   required=False
                                                   )
        
        
    def save(self, *args, **kwargs):
        alert_prefs = []
        for backend in self.backends:
            for alert in self.alerts:
                attr = self._field_id(alert, backend)
    
                alert_pref, created = AlertPreference.objects.get_or_create(
                                                user=self.user,
                                                alert_type=alert,
                                                backend=backend
                                                )
                alert_pref.preference = self.cleaned_data[attr]
                alert_pref.save()
                
                alert_prefs.append(alert_pref)
        
        return alert_prefs
            
        
            
            
    def _field_id(self, alert_type, backend):
        return "%s__%s" % (alert_type, backend)
    


class UnsubscribeForm(AlertPreferenceForm):
    """
    This form does not show any check boxes, it expects to be placed into
    the page with only a submit button and some text explaining that by
    clicking the submit button the user will unsubscribe form the Alert
    notifications. (the alert that is passed in)
    """
    
    def __init__(self, user, *args, **kwargs):
        return super(UnsubscribeForm, self).__init__(user, *args, **kwargs)
        
        for backend in self.backends:
            for alert in self.alerts:
                field_id = self._field_id(alert, backend)
                self.fields[field_id].widget = forms.HiddenInput()
                self.fields[field_id].initial = False
    
    
    def save(self, *args, **kwargs):
        alert_prefs = super(UnsubscribeForm, self).save(*args, **kwargs)
        affected_alerts = Alert.objects.filter(
                                               is_sent=False,
                                               user=self.user,
                                               backend__in=self.backends.keys(),
                                               alert_type__in=self.alerts.keys()
                                               )
        affected_alerts.delete()
        return alert_prefs
        
            
                
            



from django.contrib import admin
from models import Alert, AlertPreference



class AlertAdmin(admin.ModelAdmin):
    list_display = ('user', 'title', 'backend', 'alert_type', 'failed', 'is_sent', 'created',)
    list_filter = ('alert_type', 'backend', 'is_sent', 'failed')
    search_fields = ('=user__username', '=user__email')


class AlertPrefAdmin(admin.ModelAdmin):
    list_display = ("user", 'alert_type', "backend", 'preference')
    list_filter = ('alert_type', 'backend', 'preference')
    search_fields = ('=user__username', '=user__email')
    actions = ['subscribe', 'unsubscribe']
    
    def unsubscribe(self, request, qs): 
        qs.update(preference=False)
    unsubscribe.short_description = 'Set selected preferences to "Unsubscribed"'
    
    def subscribe(self, request, qs): 
        qs.update(preference=True)
    subscribe.short_description = 'Set selected preferences to "Subscribed"'

    

admin.site.register(Alert, AlertAdmin)
admin.site.register(AlertPreference, AlertPrefAdmin)


from django.db.models.signals import post_init, post_save
from alert.models import AlertPreference
from alert.signals import preference_updated
from alert.utils import ALERT_TYPE_CHOICES, ALERT_BACKEND_CHOICES, ALERT_TYPES, ALERT_BACKENDS

def alertpref_post_init(instance, **kwargs):
    instance._current_pref = instance.preference

def alertpref_post_save(instance, **kwargs):
    if instance._current_pref != instance.preference:
        preference_updated.send(
                                sender=instance.alert_type_obj, 
                                user=instance.user, 
                                preference=instance.preference, 
                                instance=instance
                                )

post_init.connect(alertpref_post_init, sender=AlertPreference)
post_save.connect(alertpref_post_save, sender=AlertPreference)







from datetime import datetime, timedelta
from django.core.management.base import BaseCommand
from django.core.cache import cache
from alert.models import Alert


class CacheRequiredError(Exception): pass


class Command(BaseCommand):
    help = "Send pending alerts"
    
    _cache_key = 'currently_sending_alerts'
    
    def handle(self, *args, **kwargs):
        cache.set("_dummy_cache_key", True, 60)
        if not cache.get("_dummy_cache_key", False): 
            raise CacheRequiredError
        
        if cache.get(self._cache_key, False):
            return
        
        one_day = 60*60*24
        cache.set(self._cache_key, True, one_day)
        
        alerts = Alert.pending.all()
        [alert.send() for alert in alerts]
        
        cache.delete(self._cache_key)
        


__doc__ = """
Send alerts, notifications, and messages based on events in your django application.

See the README file for details, usage info, and a list of gotchas.
"""

from setuptools import setup

setup(
    name='django-alert',
    version='0.2.1',
    author='James Robert',
    author_email='jiaaro@gmail.com',
    description=('Send alerts, notifications, and messages based '
                'on events in your django application'),
    license='MIT',
    keywords='django alerts notifications social',
    url='https://github.com/jiaaro/django-alert/',
    packages=['alert', 'alert.management', 'alert.management.commands'],
    long_description=__doc__,
    classifiers=[
    	'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Framework :: Django',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Utilities'
    ]
)



from collections import defaultdict
from datetime import datetime, timedelta
from django.db.models.signals import post_save
from django.contrib.auth.models import User

from alert.utils import BaseAlert

from example_news_app.models import NewsItem



class WelcomeAlert(BaseAlert):
    title = 'Welcome Users'
    description = 'When a user signs up, send them a nice welcome email.'
    
    signal = post_save
    sender = User
    
    # by default all users will receive this alert
    default = True
    
    def before(self, created, **kwargs):
        # if created is false, no alert will be sent
        return created
    
    def get_applicable_users(self, instance, **kwargs):
        return instance



class NewsAlert(BaseAlert):
    title = 'News'
    description = 'Get notified as soon as news updates go out.'
    
    signal = post_save
    sender = NewsItem
    
    # by default users will not receive this alert (i.e. it's opt-in)
    default = False
    
    def before(self, created, **kwargs):
        return created
    
    def get_applicable_users(self, **kwargs):
        # this alert applies to all users (but this queryset will be
        # filtered based on the users' respective alert preferences
        return User.objects.all()



# When a user signs up, one way to keep them engaged is to contact them
# periodically over the next month
#
# The easiest way to do this is to schedule all their emails in advance (we'll
# do 3 in this example:
#
#  - 3 days after signup
#  - 7 days after signup
#  - 30 days after signup 

class MarketingDrip1(BaseAlert):
    """
    first alert in the marketing drip (3 days after signup)
    
    templates...
    
    subject line: 
    "alerts/MarketingDrip1/EmailBackend/title.txt"
    
    body text:
    "alerts/MarketingDrip1/EmailBackend/body.txt"
    
    html version:
    "alerts/MarketingDrip1/EmailBackend/body.html"
    """
    title = "Trickle"
    description = "Send scheduled marketing emails to users once they give their email"
    
    # don't send on any backends except email
    default = defaultdict(lambda: False)
    default['EmailBackend'] = True
    
    signal = post_save
    sender = User
    
    def before(self, created, **kwargs):
        return created
    
    def get_applicable_users(self, instance, **kwargs):
        return instance

    def get_send_time(self, **kwargs):
        return datetime.now + timedelta(days=3)
    


class MarketingDrip2(MarketingDrip1):
    """
    second alert in the marketing drip (7 days after signup)
    
    We subclass MarketingDrip1 to avoid re-writing all that logic
    
    templates...
    
    subject line: 
    "alerts/MarketingDrip2/EmailBackend/title.txt"
    
    body text:
    "alerts/MarketingDrip2/EmailBackend/body.txt"
    
    html version:
    "alerts/MarketingDrip2/EmailBackend/body.html"
    """

    def get_send_time(self, **kwargs):
        return datetime.now + timedelta(days=7)
    

class MarketingDrip3(MarketingDrip1):
    """
    third alert in the marketing drip (30 days after signup)
    
    We could subclass MarketingDrip1 or MarketingDrip2 at this point. I'm
    using MarketingDrip1 to keep things consistent.
    
    templates...
    
    subject line: 
    "alerts/MarketingDrip3/EmailBackend/title.txt"
    
    body text:
    "alerts/MarketingDrip3/EmailBackend/body.txt"
    
    html version:
    "alerts/MarketingDrip4/EmailBackend/body.html"
    """

    def get_send_time(self, **kwargs):
        return datetime.now + timedelta(days=30)

    
    